@@ -3,7 +3,7 @@ import datetime
 import logging
 import sys
 from pathlib import Path
-from typing import List, Type
+from typing import List, Set, Type
 from unittest.mock import Mock, call, create_autospec
 
 import freezegun
@@ -16,6 +16,7 @@ from security_constraints.common import (
     SecurityConstraintsError,
     SecurityVulnerability,
     SecurityVulnerabilityDatabaseAPI,
+    SeverityLevel,
 )
 from security_constraints.main import (
     are_constraints_pip_friendly,
@@ -126,16 +127,37 @@ def test_get_args(monkeypatch) -> None:
     assert args is mock.return_value.parse_args.return_value
 
 
-def test_get_config__no_file() -> None:
-    assert get_config(config_file=None) == Configuration()
+def test_get_config__no_file(monkeypatch, arg_namespace) -> None:
+    mock_configuration_cls = create_autospec(Configuration, instance=False)
+    monkeypatch.setattr(
+        "security_constraints.main.Configuration", mock_configuration_cls
+    )
+    arg_namespace.config = None
+    assert (
+        get_config(args=arg_namespace) is mock_configuration_cls.from_args.return_value
+    )
+    mock_configuration_cls.from_args.assert_called_once_with(arg_namespace)
 
 
-def test_get_config__config_file(tmp_path) -> None:
-    config_file: Path = tmp_path / "sc_conf.yaml"
-    with open(config_file, mode="w") as fh:
-        yaml.safe_dump({"ignore_ids": ["GHSA-1", "GHSA-3"]}, fh)
-    assert get_config(config_file=str(config_file)) == Configuration(
-        ignore_ids=["GHSA-1", "GHSA-3"]
+def test_get_config__config_file(tmp_path, monkeypatch, arg_namespace) -> None:
+    mock_configuration_cls = create_autospec(Configuration, instance=False)
+    monkeypatch.setattr(
+        "security_constraints.main.Configuration", mock_configuration_cls
+    )
+    mock_config_file: Path = tmp_path / "sc_conf.yaml"
+    mock_config_file_content = {
+        "ignore_ids": ["GHSA-1", "GHSA-3"],
+        "min_severity": "HIGH",
+    }
+    arg_namespace.config = str(mock_config_file)
+    with open(mock_config_file, mode="w") as fh:
+        yaml.safe_dump(mock_config_file_content, fh)
+    assert get_config(args=arg_namespace) is mock_configuration_cls.merge.return_value
+    mock_configuration_cls.from_args.assert_called_once_with(arg_namespace)
+    mock_configuration_cls.from_dict.assert_called_once_with(mock_config_file_content)
+    mock_configuration_cls.merge.assert_called_once_with(
+        mock_configuration_cls.from_dict.return_value,
+        mock_configuration_cls.from_args.return_value,
     )
 
 
@@ -149,7 +171,11 @@ def test_setup_logging(monkeypatch, debug: bool) -> None:
     )
 
 
-def test_fetch_vulnerabilities() -> None:
+@pytest.mark.parametrize(
+    "severities",
+    [{SeverityLevel.CRITICAL}, {SeverityLevel.CRITICAL, SeverityLevel.HIGH}],
+)
+def test_fetch_vulnerabilities(severities: Set[SeverityLevel]) -> None:
     mock_vulnerabilities = [create_autospec(SecurityVulnerability) for _ in range(3)]
     mock_apis = [
         Mock(
@@ -161,7 +187,11 @@ def test_fetch_vulnerabilities() -> None:
             get_vulnerabilities=Mock(return_value=[mock_vulnerabilities[2]]),
         ),
     ]
-    assert fetch_vulnerabilities(mock_apis) == mock_vulnerabilities
+    assert (
+        fetch_vulnerabilities(mock_apis, severities=severities) == mock_vulnerabilities
+    )
+    for mock_api in mock_apis:
+        mock_api.get_vulnerabilities.assert_called_once_with(severities=severities)
 
 
 @pytest.mark.parametrize(
@@ -200,7 +230,7 @@ def test_fetch_vulnerabilities() -> None:
                     vulnerable_range="< 3.0dev1",
                 ),
             ],
-            Configuration(ignore_ids=["GHSA-X1", "GHSA-X2"]),
+            Configuration(ignore_ids={"GHSA-X1", "GHSA-X2"}),
             [
                 SecurityVulnerability(
                     name="CVE-1",
@@ -299,19 +329,20 @@ def test_sort_vulnerabilities(
             Configuration(),
             (
                 "# Generated by security-constraints x.y.z"
-                " on 1986-04-09T12:11:10.000009\n"
+                " on 1986-04-09T12:11:10.000009Z\n"
                 "# Data sources: FakeDB\n"
-                r"# Configuration: {'ignore_ids': []}"
+                r"# Configuration: {'ignore_ids': [], 'min_severity': 'CRITICAL'}"
             ),
         ),
         (
             ["FakeDB", "Another DB"],
-            Configuration(ignore_ids=["GHSA-1", "GHSA-2"]),
+            Configuration(ignore_ids={"GHSA-1", "GHSA-2"}),
             (
                 "# Generated by security-constraints x.y.z"
-                " on 1986-04-09T12:11:10.000009\n"
+                " on 1986-04-09T12:11:10.000009Z\n"
                 "# Data sources: FakeDB, Another DB\n"
-                r"# Configuration: {'ignore_ids': ['GHSA-1', 'GHSA-2']}"
+                r"# Configuration: "
+                "{'ignore_ids': ['GHSA-1', 'GHSA-2'], 'min_severity': 'CRITICAL'}"
             ),
         ),
         (
@@ -319,9 +350,9 @@ def test_sort_vulnerabilities(
             Configuration(),
             (
                 "# Generated by security-constraints x.y.z"
-                " on 1986-04-09T12:11:10.000009\n"
+                " on 1986-04-09T12:11:10.000009Z\n"
                 "# Data sources: \n"
-                r"# Configuration: {'ignore_ids': []}"
+                r"# Configuration: {'ignore_ids': [], 'min_severity': 'CRITICAL'}"
             ),
         ),
     ],
@@ -392,7 +423,7 @@ def test_format_constraints_file_line__package_mismatch() -> None:
 
 
 @pytest.mark.parametrize("to_stdout", [True, False])
-def test_main(monkeypatch, to_stdout: bool) -> None:
+def test_main(monkeypatch, arg_namespace, to_stdout: bool) -> None:
     mock_stream = Mock(isatty=Mock(return_value=to_stdout))
     mock_vulnerabilities = [create_autospec(SecurityVulnerability) for _ in range(3)]
     mock_sorted_vulnerabilities = [
@@ -446,11 +477,13 @@ def test_main(monkeypatch, to_stdout: bool) -> None:
         mock_are_constraints_pip_friendly,
     )
 
-    mock_get_args.return_value.version = False
-    mock_get_args.return_value.output = mock_stream
-    mock_get_args.return_value.dump_config = False
-    mock_get_args.return_value.ignore_ids = ["GHSA-X1"]
-    mock_get_config.return_value = Configuration(ignore_ids=["GHSA-X2"])
+    mock_get_args.return_value = arg_namespace
+    arg_namespace.output = mock_stream
+    arg_namespace.ignore_ids = ["GHSA-X1"]
+    arg_namespace.min_severity = SeverityLevel.CRITICAL
+    mock_get_config.return_value = Configuration(
+        ignore_ids={"GHSA-X1", "GHSA-X2"}, min_severity=SeverityLevel.CRITICAL
+    )
     mock_get_apis.return_value = [mock_api]
     mock_fetch_vulnerabilities.return_value = mock_vulnerabilities
     mock_sort_vulnerabilities.return_value = mock_sorted_vulnerabilities
@@ -466,14 +499,16 @@ def test_main(monkeypatch, to_stdout: bool) -> None:
     assert exit_code == 0
     mock_yaml_dump.assert_not_called()
     mock_get_args.assert_called_once_with()
-    mock_setup_logging.assert_called_once_with(debug=mock_get_args.return_value.debug)
-    mock_get_config.assert_called_once_with(
-        config_file=mock_get_args.return_value.config
-    )
+    mock_setup_logging.assert_called_once_with(debug=arg_namespace.debug)
+    mock_get_config.assert_called_once_with(args=arg_namespace)
     mock_get_apis.assert_called_once_with()
-    mock_fetch_vulnerabilities.assert_called_once_with([mock_api])
+    mock_fetch_vulnerabilities.assert_called_once_with(
+        [mock_api], severities={SeverityLevel.CRITICAL}
+    )
     mock_filter_vulnerabilities.assert_called_once_with(
-        config=Configuration(ignore_ids=["GHSA-X2", "GHSA-X1"]),
+        config=Configuration(
+            ignore_ids={"GHSA-X2", "GHSA-X1"}, min_severity=SeverityLevel.CRITICAL
+        ),
         vulnerabilities=mock_vulnerabilities,
     )
     mock_sort_vulnerabilities.assert_called_once_with(
@@ -505,7 +540,7 @@ def test_main(monkeypatch, to_stdout: bool) -> None:
 
 
 @pytest.mark.parametrize("to_stdout", [True, False])
-def test_main__dump_config(monkeypatch, to_stdout: bool) -> None:
+def test_main__dump_config(arg_namespace, monkeypatch, to_stdout: bool) -> None:
     mock_stream = Mock(isatty=Mock(return_value=to_stdout))
     mock_get_args = create_autospec(get_args)
     mock_setup_logging = create_autospec(setup_logging)
@@ -521,23 +556,25 @@ def test_main__dump_config(monkeypatch, to_stdout: bool) -> None:
         mock_get_apis,
     )
 
-    mock_get_args.return_value.version = False
-    mock_get_args.return_value.output = mock_stream
-    mock_get_args.return_value.dump_config = True
-    mock_get_args.return_value.ignore_ids = ["GHSA-X1"]
-    mock_get_config.return_value = Configuration(ignore_ids=["GHSA-X2"])
+    mock_get_args.return_value = arg_namespace
+    arg_namespace.output = mock_stream
+    arg_namespace.dump_config = True
+    arg_namespace.ignore_ids = ["GHSA-X1"]
+    arg_namespace.min_severity = SeverityLevel.CRITICAL
+    mock_get_config.return_value = Configuration(
+        ignore_ids={"GHSA-X1", "GHSA-X2"}, min_severity=SeverityLevel.CRITICAL
+    )
 
     exit_code = main()
 
     assert exit_code == 0
     mock_yaml_dump.assert_called_once_with(
-        {"ignore_ids": ["GHSA-X2", "GHSA-X1"]}, stream=sys.stdout
+        {"ignore_ids": ["GHSA-X1", "GHSA-X2"], "min_severity": "CRITICAL"},
+        stream=sys.stdout,
     )
     mock_get_args.assert_called_once_with()
     mock_setup_logging.assert_called_once_with(debug=mock_get_args.return_value.debug)
-    mock_get_config.assert_called_once_with(
-        config_file=mock_get_args.return_value.config
-    )
+    mock_get_config.assert_called_once_with(args=mock_get_args.return_value)
     mock_get_apis.assert_not_called()
     mock_stream.write.assert_not_called()
     mock_stream.isatty.assert_called_once_with()
@@ -547,7 +584,7 @@ def test_main__dump_config(monkeypatch, to_stdout: bool) -> None:
         mock_stream.close.assert_called_once_with()
 
 
-def test_main__version(monkeypatch, capsys) -> None:
+def test_main__version(monkeypatch, capsys, arg_namespace) -> None:
     mock_version = Mock(return_value="x.y.z")
     monkeypatch.setattr("security_constraints.main.version", mock_version)
     mock_stream = Mock()
@@ -565,11 +602,11 @@ def test_main__version(monkeypatch, capsys) -> None:
         mock_get_apis,
     )
 
-    mock_get_args.return_value.version = True
-    mock_get_args.return_value.output = mock_stream
-    mock_get_args.return_value.dump_config = True
-    mock_get_args.return_value.ignore_ids = ["GHSA-X1"]
-    mock_get_config.return_value = Configuration(ignore_ids=["GHSA-X2"])
+    mock_get_args.return_value = arg_namespace
+    arg_namespace.version = True
+    arg_namespace.output = mock_stream
+    arg_namespace.dump_config = True
+    arg_namespace.ignore_ids = ["GHSA-X1"]
 
     exit_code = main()
 
@@ -591,7 +628,7 @@ def test_main__version(monkeypatch, capsys) -> None:
     [(SecurityConstraintsError, 1), (Exception, 2)],
 )
 def test_main__exception(
-    monkeypatch, exception_type: Type[Exception], expected_exit_code: int
+    monkeypatch, arg_namespace, exception_type: Type[Exception], expected_exit_code: int
 ) -> None:
     mock_stream = Mock(isatty=Mock(return_value=True))
     mock_get_args = create_autospec(get_args)
@@ -614,11 +651,10 @@ def test_main__exception(
         "security_constraints.main.filter_vulnerabilities", mock_filter_vulnerabilities
     )
 
-    mock_get_args.return_value.version = False
-    mock_get_args.return_value.output = mock_stream
-    mock_get_args.return_value.dump_config = False
-    mock_get_args.return_value.ignore_ids = ["GHSA-X1"]
-    mock_get_config.return_value = Configuration(ignore_ids=["GHSA-X2"])
+    mock_get_args.return_value = arg_namespace
+    arg_namespace.output = mock_stream
+    arg_namespace.ignore_ids = ["GHSA-X1"]
+    mock_get_config.return_value = Configuration(ignore_ids={"GHSA-X1", "GHSA-X2"})
 
     exit_code = main()
 
@@ -626,19 +662,17 @@ def test_main__exception(
     mock_yaml_dump.assert_not_called()
     mock_get_args.assert_called_once_with()
     mock_setup_logging.assert_called_once_with(debug=mock_get_args.return_value.debug)
-    mock_get_config.assert_called_once_with(
-        config_file=mock_get_args.return_value.config
-    )
+    mock_get_config.assert_called_once_with(args=arg_namespace)
     mock_get_apis.assert_called_once_with()
     mock_filter_vulnerabilities.assert_not_called()
     mock_stream.write.assert_not_called()
     mock_stream.isatty.assert_called_once_with()
 
 
-def test_main__output_none_exception(monkeypatch) -> None:
+def test_main__output_none_exception(monkeypatch, arg_namespace) -> None:
     mock_get_args = create_autospec(get_args)
-    mock_get_args.return_value.version = False
-    mock_get_args.return_value.output = None
+    mock_get_args.return_value = arg_namespace
+    arg_namespace.output = None
     mock_setup_logging = create_autospec(setup_logging)
     mock_get_config = create_autospec(get_config)
     monkeypatch.setattr("security_constraints.main.get_args", mock_get_args)
